<?php

namespace June\Controller\Api;

use June\Components\Logger\Helper\LoggerHelper;
//use Shopware\Core\Framework\Api\Acl\AclCriteriaValidator;
//use Shopware\Core\Framework\Api\Converter\ApiVersionConverter;
use Shopware\Core\Framework\Context;
//use Shopware\Core\Content\Media\File\FileSaver;
//use Shopware\Core\Framework\DataAbstractionLayer\DefinitionInstanceRegistry;
//use Shopware\Core\Framework\DataAbstractionLayer\EntityProtection\EntityProtectionValidator;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
//use Shopware\Core\Framework\DataAbstractionLayer\Search\CompositeEntitySearcher;
//use Shopware\Core\Framework\DataAbstractionLayer\Search\RequestCriteriaBuilder;
//use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\Framework\Uuid\Uuid;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
//use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
//use Symfony\Component\Serializer\Serializer;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\SystemConfig\SystemConfigService;


#[Route(defaults: ['_routeScope' => ['api']])]
class PageController extends AbstractController
{

    const JUNE_AUTH_USERNAME = 'June.config.username';
    const JUNE_AUTH_PASSWORD = 'June.config.password';

    private EntityRepository $junePageRepository;

    private SystemConfigService $configService;

    private LoggerHelper $loggerHelper;

    /**
     * @var Context
     */
    private Context $context;

    public function __construct
    (
        EntityRepository $junePageRepository,
        SystemConfigService $configService,
        LoggerHelper $loggerHelper
    )
    {
        $this->junePageRepository = $junePageRepository;
        $this->configService = $configService;
        $this->loggerHelper = $loggerHelper;
    }

    #[Route('/api/junecms', name: 'api.extension.create_page', defaults: ['auth_required'=>false], methods: ['POST'])]
    public function junepage(Request $request): JsonResponse
    {
        if($request->headers->get('authorization')===$this->getJuneAuthToken()) {
            try {
                $context = Context::createDefaultContext();
                $id = Uuid::randomHex();
                $content = $request->toArray();
                //$content = json_decode($content, true);

                if ($content && !empty($content['data']) && isset($content['data']['js'])) {
                    $js = $this->createJsFile($content['data']['js'], $content['data']['js_libs']);
                    $css = $this->createCssFile($content['data']['css'], $content['data']['css_libs']);

                    $data = [
                        'id' => $id,
                        'sitemapid' => $content['id'],
                        'sitemapnodename' => $content['data']['title'],
                        'sitemapnodeid' => $content['sitemap_id'],
                        'html' => $content['data']['html'],
                        'css' => $css,
                        'js' => $js,
                    ];

                    $criteria = new Criteria();
                    $criteria->addFilter(new EqualsFilter('sitemapid', $content['id']));

                    if ($this->junePageRepository->search($criteria, $context)->count() > 0) {
                        $result = $this->junePageRepository->search($criteria, $context)->first();
                        $result = $result->id;
                        $this->updatePage($data, $context, $result);

                        return new JsonResponse($data, Response::HTTP_OK);
                    } else {
                        $this->createPage($data, $context);

                        return new JsonResponse($data, Response::HTTP_OK);
                    }
                } else {
                    return new JsonResponse("Malformed data transmitted. data:".var_export(!empty($content['data']), true)." js:".var_export(isset($content['data']['js']), 1)." css:".var_export(isset($content['data']['css']), true), Response::HTTP_BAD_REQUEST);
                }
            } catch (\Exception $exception) {
                $this->loggerHelper->logError($exception->getMessage(), "publishing");

                return new JsonResponse($exception->getMessage(), Response::HTTP_BAD_REQUEST);
            }
        } else {
            return new JsonResponse("Unauthorized", Response::HTTP_UNAUTHORIZED);
        }
    }

    private function createPage($data, $context) {
        $this->junePageRepository->create([$data], $context);
    }

    private function updatePage($data, $context, $id) {
        $data['id'] = $id;
        $this->junePageRepository->update([$data], $context);
    }

    private function createJsFile($js, $js_libs)
    {
        $jsAssets = "";
        $jsLibs = "";
        if (count($js) > 0) {
            foreach ($js as $k => $v) {
                $jsAssets = $jsAssets . file_get_contents($v) . ";; \n \n \n ";
            }
            $fileName = 'js/'. $this->getFileHash($jsAssets) . '.js';
        } else {
            $fileName = 'js/' . $this->getFileHash($js_libs) . '.js';
        }
        $libs = explode(';;', $js_libs);

        foreach ($libs as $k) {
            if(!empty($k )){
                $jsLibs = $jsLibs . file_get_contents($k) . ";; \n \n \n ";
            }
        }

        $fileCode = $jsLibs . "; \n \n \n" . $jsAssets;
        file_put_contents($fileName, $fileCode);
        return '/'.$fileName;
    }

    private function createCssFile($css, $css_libs)
    {
        $fileName = 'css/' . $this->getFileHash($css) . '.css';
        $fileCode = file_get_contents($css_libs) . "\n\n /* CSS FILE SPLIT */\n \n" . $css;
        file_put_contents($fileName, $fileCode);
        return '/'.$fileName;
    }

    private function getFileHash($filename)
    {
        return md5($filename);
    }

    private function getJuneAuthToken() {
        $username = $this->configService->get(self::JUNE_AUTH_USERNAME);
        $password = $this->configService->get(self::JUNE_AUTH_PASSWORD);

        return 'Basic '.base64_encode($username.':'.$password);
    }

}
