import './list-token-detail.scss';
import template from './list-token-detail.html.twig';

const { Component, Mixin } = Shopware;
const { Criteria } = Shopware.Data;

Component.register('june-token-detail', {
    template,

    inject: [
        'repositoryFactory',
    ],

    shortcuts: {
        'SYSTEMKEY+S': 'onSave',
        ESCAPE: 'onCancel'
    },

    data() {
        return {
            isLoading: false,
            isSaveSuccessful: false,
            collectToken: null,
            processSuccess: false,
        };
    },

    computed: {
        juneFlowActionRepository() {
            return this.repositoryFactory.create('juneflowaction');
        },

        defaultCriteria() {
            return new Criteria();
        },
    },

    created() {
        this.createdComponent();
    },

    methods: {
        createdComponent() {
            this.isLoading = true;

            this.juneFlowActionRepository.get(
                this.$route.params.id,
                Shopware.Context.api,
                this.defaultCriteria
            ).then((result) => {
                this.collectToken = result;
                this.isLoading = false;
            });
        },

        saveFinish() {
            this.isSaveSuccessful = false;
        },

        async onSave() {
            this.isLoading = true;
            this.isSaveSuccessful = false;

            return this.juneFlowActionRepository.save(this.collectToken, Shopware.Context.api).then(() => {
                this.isLoading = false;
                this.isSaveSuccessful = true;
                this.$router.push({ name: 'june.cms.token' });
            })
        },

        onCancel() {
            this.$router.push({ name: 'june.cms.token' });
        },
    }
});
